package jpa20.war.bvcallbackbmt.session;

import javax.annotation.Resource;
import javax.ejb.*;
import javax.persistence.*;
import javax.transaction.*;
import java.util.*;

import jpa20.war.bvcallbackbmt.entity.*;

@Stateless
@TransactionManagement(TransactionManagementType.BEAN)
public class TestEJB {
    
    @PersistenceContext 
    private EntityManager em;
    @Resource
    private  UserTransaction utx;

    public TestEJB() {
    }

    // initData
    public boolean test1() {
        boolean pass= false;

	Employee e1 = new Employee(1, "emp1", 1000);
	Employee e2 = new Employee(2, "emp2", 2000);
	Employee e3 = new Employee(3, "emp3", 3000);
	Employee e4 = new Employee(4, "emp4", 4000);
	Project p1 = new Project(1, "proj1");
	Collection<Employee> employees = new ArrayList<Employee>();
	employees.add(e1);
	employees.add(e2);
	employees.add(e3);
	employees.add(e4);
	p1.setEmployees(employees);
     
	// Persist Cascade without long name
	try {
	  System.out.println("1. Persisting project....");
          utx.begin();
	  em.persist(p1);
          utx.commit();
	  pass = true;
        } catch(Throwable e){
	  e.printStackTrace();
        }
        return pass;
    }

    // persist Employee with a long name
    public boolean test2() {
        boolean pass= false;
	Employee e5 = new Employee(5, "myLongName5", 5000);
	try {
	  System.out.println("2. Persisting employee with long name....");
          utx.begin();
	  em.persist(e5);
          utx.commit();
	  System.out.println("Error: not get BV ex for persist");
	} catch(javax.validation.ConstraintViolationException vex){
	  System.out.println("Expected BV Ex");
	  pass= true;
	  String msg = vex.getMessage();
	  System.out.println("msg="+msg);
          try {
             System.out.println("getStatus()="+ utx.getStatus());
             // http://www.j2ee.me/javaee/5/docs/api/javax/transaction/Status.html
             // Constant Field Values
             // 1 -- STATUS_MARKED_ROLLBACK
             if ( utx.getStatus() == 1) {
	       utx.rollback();
  	       System.out.println("done with rollback ");
	     }
          } catch (javax.transaction.SystemException txex){
   	    System.out.println("Got Tx Ex");
	  }
        } catch(Throwable e){
	  System.out.println("Unexpected Ex");
	  // e.printStackTrace();
        }
        return pass;
    }


    // update Employee with a long name
    public boolean test3() {
        boolean pass= false;
	try {
	  System.out.println("3. Updating employee with long name....");
          utx.begin();
	  Employee e = em.find(Employee.class, 3);
	  e.setName("myLongName3");
          em.merge(e);
          em.flush();
          System.out.println("utx.commit ...");
          utx.commit();
	  System.out.println("Error: not get BV ex for update");

        } catch (javax.validation.ConstraintViolationException vex) {
	  System.out.println("Expected BV Ex");
	  pass= true;
	  String msg = vex.getMessage();
	  System.out.println("msg="+msg);
          try {
             System.out.println("getStatus()="+ utx.getStatus());
             // 1 -- STATUS_MARKED_ROLLBACK
             if ( utx.getStatus() == 1) {
	       utx.rollback();
               System.out.println("done with rollback ");
	     }
	     
          } catch (javax.transaction.SystemException txex){
   	    System.out.println("Got Tx Ex");
	  }
	} catch(Throwable e){
          System.out.println("Upexpected Trowable e");
	  // e.printStackTrace();
        }
        return pass;
    }

    // remove Employee with a long name
    public boolean test4() {
        boolean pass= false;
        try {
	  System.out.println("4. Removing employee with long name....");
          utx.begin();
	  Employee e = em.find(Employee.class, 1);
	  e.setName("myLongName1");
	  em.remove(e);
          utx.commit();
	  System.out.println("OK: not get BV ex for remove");
	  pass= true;
	} catch (javax.validation.ConstraintViolationException ex) {
	  System.out.println("BV Ex");
	  String msg = ex.getMessage();
	  System.out.println("msg="+msg);
	} catch(Throwable e){
	  System.out.println("Unexpected Ex");
	  e.printStackTrace();
        }
        return pass;
    }

    // verify previous operations
    public boolean test5() {
        boolean pass= false;
        boolean bvsize = true;
	try {
	  System.out.println("5. Verifying employee ....");
	  Employee emp = null;
	  Query q= em.createQuery("SELECT e FROM Employee e");
	  List result = q.getResultList();
	  int size = result.size();
	  for (int i = 0 ; i < size ; i++) {
	    emp = (Employee) result.get(i);          
	    String name = emp.getName();
	    System.out.println("i=" + i + ", name=" + name);
	    if (name.length() > 5) {
	      bvsize = false;
	    }
	  }
	  System.out.println("size =" +size+", bvsize="+bvsize);
	  if (size == 3 && bvsize){
	    pass = true;
	  }
	} catch(Throwable e){
	  System.out.println("Unexpected Ex");
	  e.printStackTrace();
        }
        return pass;
    }


}




